{{
*************************************
* File jvmStack: handling the stack *
*************************************
}}

{{
   Values are stored onto the stack using big endian.
   The jvmData methods writeJavaIntBE and readJavaIntBE are used to read/write stack items.
   The stack grows upwards.
   Stackpointer points to first free stack byte.
}}


OBJ
  jd: "jvmData"

  
PUB jb_pushframe(method) | size,retaddr,oldfp,oldsp,oldmp
{{ Auxiliary function (not bytecode): Meaning: Push new frame onto the stack. }}
  size := jd.readJavaByte(method)   'number of parameters
  size += jd.readJavaByte(method+1) 'number of locals
  retaddr := jd.getJvmPC+3
  oldfp := jd.getJvmFP
  oldsp := jd.getJvmSP
  oldmp := jd.getJvmMP
  jd.setJvmFP(oldsp)
  jd.setJvmSP(oldsp + size + 6)
  jd.setJvmMP(method)
  jd.writeJavaIntBE(oldsp + size,oldmp)
  jd.writeJavaIntBE(oldsp + size + 2,oldfp)
  jd.writeJavaIntBE(oldsp + size + 4,retaddr) 
  

PUB jb_popframe: value | size,oldmp,oldfp,oldsp,retaddr
{{ Auxiliary function (not bytecode): Meaning: Pop frame from the stack. }}
  size := jd.readJavaByte(jd.getJvmMP)   'number of parameters
  size += jd.readJavaByte(jd.getJvmMP+1) 'number of locals
  oldmp := jd.readJavaIntBE(jd.getJvmFP + size)
  oldfp := jd.readJavaIntBE(jd.getJvmFP + size + 2)
  retaddr := jd.readJavaIntBE(jd.getJvmFP + size + 4)
  jd.setJvmFP(oldfp)
  jd.setJvmMP(oldmp)
  jd.setJvmSP(oldsp)
  jd.setJvmPC(retaddr)
  'what about return value ??
  

PUB jb_push(value) | sp
{{ Auxiliary function (not bytecode): Meaning: Push word onto the stack. }}
  sp := jd.getJvmSP
  jd.writeJavaIntBE(sp,value)
  jd.setJvmSP(sp+2)
  

PUB jb_nop
{{ Meaning: Do nothing. }}
{{
  JEM format: nop
}}
  jd.incJvmPC(1)
  

PUB jb_pop: value | sp
{{ Meaning: Pop the top word from the stack. }}
{{
  JEM format: pop
}}
  sp := jd.getJvmSP - 2
  jd.setJvmSP(sp)
  jd.incJvmPC(1)
  return jd.readJavaIntBE(sp)
  

PUB jb_pop2: value | w1,w2
{{ Meaning: Pop the top two words from the stack. }}
{{
  JEM format: pop2
}}
  w1 := jb_pop  'lsw
  w2 := jb_pop  'msw
  jd.incJvmPC(1)
  return (w2 << 16) | w1  'return 32bits result

  
PUB jb_dup | w1
{{ Meaning: Duplicate the top word on the stack. }}
{{
  JEM format: dup
}}
  w1 := jb_pop
  jb_push(w1)
  jb_push(w1)
  jd.incJvmPC(1)
  

PUB jb_dup2 | w1,w2
{{ Meaning: Duplicate the top two words on the stack. }}
{{
  JEM format: dup2
}}
  w1 := jb_pop
  w2 := jb_pop
  jb_push(w2)
  jb_push(w1)
  jb_push(w2)
  jb_push(w1)
  jd.incJvmPC(1)
  
  
PUB jb_dup_x1 | w1,w2
{{ Meaning: Duplicate the top word on the stack and insert a copy two words down in the stack. }}
{{
  JEM format: dup_x1
}}
  w1 := jb_pop
  w2 := jb_pop
  jb_push(w1)
  jb_push(w2)
  jb_push(w1)
  jd.incJvmPC(1)
  
  
PUB jb_dup2_x1 | w1,w2,w3
{{ Meaning: Duplicate the top two words on the stack and insert the copies two words down in the stack. }}
{{
  JEM format: dup2_x1
}}
  w1 := jb_pop
  w2 := jb_pop
  w3 := jb_pop
  jb_push(w2)
  jb_push(w1)
  jb_push(w3)
  jb_push(w2)
  jb_push(w1)
  jd.incJvmPC(1)
  

PUB jb_dup_x2 | w1,w2,w3
{{ Meaning: Duplicate the top word on the stack and insert the copy three words down in the stack. }}
{{
  JEM format: dup_x2
}}
  w1 := jb_pop
  w2 := jb_pop
  w3 := jb_pop
  jb_push(w1)
  jb_push(w3)
  jb_push(w2)
  jb_push(w1)
  jd.incJvmPC(1)
  

PUB jb_dup2_x2 | w1,w2,w3,w4
{{ Meaning: Duplicate the top two words on the stack and insert the copies three words down in the stack. }}
{{
  JEM format: dup2_x2
}}
  w1 := jb_pop
  w2 := jb_pop
  w3 := jb_pop
  w4 := jb_pop
  jb_push(w2)
  jb_push(w1)
  jb_push(w4)
  jb_push(w3)
  jb_push(w2)
  jb_push(w1)
  jd.incJvmPC(1)
  

PUB jb_swap | w1,w2
{{ Meaning: Swap the top two elements on the stack. }}
{{
  JEM format: swap
}}
  w1 := jb_pop
  w2 := jb_pop
  jb_push(w1)
  jb_push(w2)
  jd.incJvmPC(1)
  
       