{{
*******************************************************
* File jvmSubReturn: handling subroutines and returns *
*******************************************************
}}

{{
  Instructions that jump to a subroutine push the return address onto the stack.
  This address is retrieved from a local variable.

  Returning from Methods.  
  Returns are handled in one of two ways by bytecode instructions: return (void) or return (normal). 
  Void returns are used to back out cleanly from the previous method. Thus, all values on the operand stack 
  are discarded and the interpreter then returns control to its caller. Normal returns are used to proceed 
  normally with execution and push a value associated with the previous method onto the stack. After the 
  return value is pushed onto the stack, any other values on the operand stack are discarded and the 
  interpreter then returns control to its caller. 
  Note:
  Java's operand stack is not contiguous like the operand stacks you may be familiar with from programming
  in other languages. Because each method has its own section of the operand stack, when the operand stack
  for the method is discarded, only the section related to the method is cleared out. 
}}


OBJ
  js: "jvmStack"
  jd: "jvmData"
  
  
{{PUB jb_jsr}}
{{ Meaning: Jump subroutine. }}
{{
  Description: Execution proceeds at the signed 16-bit offset from the address of this instruction.
  The offset is constructed from byte1 and byte2. The address of the instruction immediately following
  the current instruction is pushed onto the stack to provide a return address from the subroutine.
}}


{{PUB jb_jsr_w}}
{{ Meaning: Jump subroutine for wide index. }}
{{
  Description: Execution proceeds at the signed 32-bit offset from the address of this instruction.
  The offset is constructed from byte1, byte2, byte3, and byte4. The address of the instruction immediately
  following the current instruction is pushed onto the stack to provide a return address from the subroutine.
}}


{{PUB jb__ret}}
{{ Meaning: Return from subroutine. }}
{{
  Description: Local variable in the current Java frame must contain a return address.
  The contents of the local variable are written into the program counter.
}}


{{PUB jb_ret_w}}
{{ Meaning: Return from subroutine for wide index. }}
{{
  Description: Local variable in the current Java frame must contain a return address.
  The contents of the local variable are written into the program counter.
}}


{{PUB jb_breakpoint}}
{{ Meaning: Breakpoint. }}
{{
  Description: Stop and pass control to breakpoint handler.
}}


PUB jb_return
{{ Meaning: Return (void) from method. }}
{{
  JEM format: return
}}


PUB jb_ireturn
{{ Meaning: Return integer from function. }}
{{
  JEM format: ireturn
}}


{{PUB jb_lreturn}}
{{ Meaning: Return long integer from function. }}


{{PUB jb_freturn}}
{{ Meaning: Return single float from function. }}


{{PUB jb_dreturn}}
{{ Meaning: Return double float from function. }}


{{PUB jb_areturn}}
{{ Meaning: Return object reference from function. }}

